/*
******************************************************************************

   Edemo extension initialization.

   Authors:	1999 Andreas Beck		[becka@ggi-project.org]
   		1999-2000 Marcus Sundberg	[marcus@ggi-project.org]
  
   This code is placed in the public domain and may be used freely for any
   purpose.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#include "config.h"
#include <ggi/internal/edemo.h>

#include <ggi/gg.h> /* We use LibGG to manage config files */


/* Static variables 
 */
static int 	 _edemoLibIsUp       = 0;
static void	*_edemoConfigHandle  = NULL;
static char      _edemoconfstub[512] = EDEMOCONFFILE;
static char     *_edemoconffile      = _edemoconfstub+EDEMOTAGLEN;

/* Extension ID. Defaulting to -1 should make segfault on abuse more likely...
   This is exported so that sublibs can use it.
 */
EXPORTVAR ggi_extid _edemoID = -1;


#define SUBLIB_PREFIX	"EDEMOdl_"


/* Dummy function which returns -1
   We use this to reset the function pointers */
static int dummyfunc(void)
{
	printf("LibEDEMO: dummyfunc() of LibEDEMO called\n");

	return -1;
}	/* dummyfunc */


/* Reset all function pointers to dummyfunc */
static void clearfuncs(edemopriv *priv)
{
	priv->function1 = (void *) dummyfunc; 
	priv->function2 = (void *) dummyfunc;
}	/* clearfuncs */


/* This function is called by LibGGI when the API-list changes */
static int changed(ggi_visual_t vis, int whatchanged)
{
	printf("LibEDEMO: changed(%p, 0x%x) of LibEDEMO called\n",
	       vis, whatchanged);

	switch (whatchanged) {
	case GGI_CHG_APILIST:
		{
		int i;
		const char *fname;
		ggi_dlhandle *lib;
		char api[GGI_MAX_APILEN], args[GGI_MAX_APILEN];

		clearfuncs(LIBGGI_EDEMOEXT(vis));

		for (i=0; ggiGetAPI(vis, i, api, args) == 0; i++) {
			strcat(api,"-edemo");
			fname = ggMatchConfig(_edemoConfigHandle, api, NULL);
			if (fname == NULL) {
				/* No special implementation for this sublib */
				continue;
			}
			lib = ggiExtensionLoadDL(vis, fname, args, NULL,
						 SUBLIB_PREFIX);
		}	/* for */
		}

		break;

	}	/* switch */
                                
	return 0;
}	/* changed */


/* Initialize the extension
 */
int edemoInit(void)
{
	int err;

	_edemoLibIsUp++;
	if (_edemoLibIsUp > 1) return 0; /* Initialize only at first call. */

	err = ggLoadConfig(_edemoconffile, &_edemoConfigHandle);
	if (err != GGI_OK) {
		fprintf(stderr,"LibEDEMO: couldn't open %s\n",
			_edemoconffile);
		_edemoLibIsUp--;
		return err;
	}	/* if */

	_edemoID = ggiExtensionRegister("EDEMO", sizeof(edemopriv), changed);

	if (_edemoID < 0) {
		fprintf(stderr,
			"LibEDEMO: failed to register as extension.\n");
		_edemoLibIsUp--;
		ggFreeConfig(_edemoConfigHandle);
		return _edemoID;
	}	/* if */

	return 0;
}	/* edemoInit */


/* Deinitialize the extension
 */
int edemoExit(void)
{
	int rc;
    
	if (!_edemoLibIsUp) return -1;

	if (_edemoLibIsUp > 1) {
		/* Exit only at last call */
		_edemoLibIsUp--;
		return 0;
	}	/* if */
	
	rc = ggiExtensionUnregister(_edemoID);
	ggFreeConfig(_edemoConfigHandle);

	_edemoLibIsUp = 0;

	return rc;
}	/* edemoExit */


/* Attach the extension to a visual
 */
int edemoAttach(ggi_visual_t vis)
{
	int rc;

	printf("LibEDEMO: edemoAttach(%p) called\n", vis);

	rc = ggiExtensionAttach(vis, _edemoID);
	if (rc == 0) {
		/* We are actually creating the primary instance. */
		memset(LIBGGI_EDEMOEXT(vis), 0, sizeof(edemopriv));
		LIBGGI_EDEMOEXT(vis)->priv = NULL;

		/* Now fake an "API change" so the right libs get loaded */
		changed(vis, GGI_CHG_APILIST);
	}	/* if */

	return rc;
}	/* edemoAttach */


/* Detach the extension from a visual
 */
int edemoDetach(ggi_visual_t vis)
{
	printf("LibEDEMO: edemoDetach(%p) called\n", vis);

	return ggiExtensionDetach(vis, _edemoID);
}	/* edemoDetach */

/* This should be included by exactly _one_ file in each extension library. */
#include <ggi/internal/ggilibinit.h>
